#include "connectioncache.hh"

const Connection &ConnectionCache::GetConnection(unsigned ConnID) const
{
    /* NOTE: If ConnID is out of range (use Has()), this causes a crash. */
    return recs[ConnID].Conn;
}

bool ConnectionCache::IsExpiring(unsigned ConnID) const
{
    return GetConnection(ConnID).Protocol != SockProtocolTCP;
}

bool ConnectionCache::Has(unsigned ConnID) const
{
    return ConnID < recs.size()
        && recs[ConnID].LastTouch > 0;
}

unsigned ConnectionCache::GetConnectionID(const Connection &conn)
{
    static const unsigned EXPIRELEN = 3600;
    time_t Now = time(NULL);
    
    map<Connection, unsigned>::iterator i = key2rec.find(conn);
    if(i != key2rec.end())
    {
        recs[i->second].LastTouch = Now;
        return i->second;
    }
    
    unsigned ConnID;
    /* Find out if there's an expiring connection */
    for(ConnID=0; ConnID<recs.size(); ++ConnID)
    {
        if(!recs[ConnID].LastTouch)break;
        
        if(IsExpiring(ConnID) && Now > recs[ConnID].LastTouch + EXPIRELEN)
        {
            DeleteConnectionID(ConnID);
            break;
        }
    }
    
    Assign(ConnID, conn);
    
    owner->AddOpeningMsg(ConnID, conn.Protocol, conn.DestPort);
    
    return ConnID;
}

void ConnectionCache::Assign(unsigned ConnID, const Connection &conn)
{
    if(ConnID >= recs.size())
    {
        recs.reserve((ConnID + 63) & ~63);
        recs.resize(ConnID + 1);
    }
    
    recs[ConnID].LastTouch = time(NULL);
    recs[ConnID].Conn      = conn;
    key2rec.insert(make_pair(conn, ConnID));
}

void ConnectionCache::TouchConnection(unsigned ConnID)
{
    if(ConnID >= recs.size())return;
    recs[ConnID].LastTouch = time(NULL);
}

void ConnectionCache::DeleteConnectionID(unsigned ConnID)
{
    if(ConnID >= recs.size())return;
    
    owner->AddClosingMsg(ConnID);
    
    map<Connection, unsigned>::iterator i = key2rec.find(recs[ConnID].Conn);
    if(i != key2rec.end())
        key2rec.erase(i);
    
    recs[ConnID].LastTouch = 0;
}

ConnectionCache::ConnectionCache(struct StdinType *o) : owner(o)
{
}

ConnectionCache::~ConnectionCache()
{
}
